unit uBezier;

interface

   uses UObjeto, UPonto, Dialogs, Classes;


   Type
     Pponto = ^aponto;
       aponto = record
         ponto : TPonto;
   end;

   type TBezier=class(TObjeto)
  private
     vnpontos : integer;
     procedure setPonto(i: integer; x, y, z: real);
   protected
     bezier : Tlist;
   public
      function ValorBezier(t: real): TPonto;
      function getPonto(i: integer): TPonto;
      procedure adicionaPonto(_x, _y, _z: integer);
      procedure translada(dx,dy,dz:real); override;
      procedure escalona(origem: boolean; sx, sy, sz: real);override;
      procedure rotacao(tipoRotacao, eixoRotacao, angulo: integer); override;
      constructor crie_se(_nome:String);
      destructor destrua_se;

   published
    property npontos: integer read vnpontos write vnpontos;

  end;

implementation

procedure TBezier.adicionaPonto(_x, _y, _z: integer);
var
 ponto: Pponto;
begin
   New(ponto);
   ponto^.ponto:=TPonto.crie_se(nome,_x,_y,_z);
   bezier.Add(ponto);
   npontos:=npontos+1;
end;

constructor TBezier.crie_se(_nome:String);
begin
   tipo:='Bezier';
   nome:=_nome;
   bezier:=TList.Create;
   npontos:=0;
end;

destructor TBezier.destrua_se;
begin
end;

function TBezier.getPonto(i: integer): TPonto;
var
 ponto : Pponto;
begin
    New(ponto);
    ponto:= bezier.items[i-1];
    result:=ponto^.ponto;
end;

procedure TBezier.setPonto(i:integer;x,y,z : real);
var
 ponto : Pponto;
begin
   New(ponto);
   ponto^.ponto:=TPonto.crie_se(nome,x,y,z);
   bezier.items[i-1]:=ponto;
end;

function TBezier.ValorBezier(t:real) : TPonto;
var t2,t3:real;
    ponto : tponto;
begin
     t2 := t * t;
     t3 := t * t2;

     ponto:= TPonto.crie_se('',0,0,0);

     ponto.x:=         (1 - 3*t + 3*t2 - t3)*getPonto(1).x;
     ponto.x :=ponto.X+(3*t - 6*t2 + 3*t3)*getPonto(2).x;
     ponto.X :=ponto.X+(3*t2 - 3*t3)*getPonto(3).x;
     ponto.X :=ponto.X+(t3)*getPonto(4).x;

     ponto.Y:=        (1 - 3*t + 3*t2 - t3)*getPonto(1).y;
     ponto.Y:=ponto.Y+(3*t - 6*t2 + 3*t3)*getPonto(2).y;
     ponto.Y:=ponto.Y+(3*t2 - 3*t3)*getPonto(3).y;
     ponto.Y:=ponto.Y+(t3)*getPonto(4).y;

     ponto.Z:=        (1 - 3*t + 3*t2 - t3)*getPonto(1).Z;
     ponto.Z:=ponto.Z+(3*t - 6*t2 + 3*t3)*getPonto(2).Z;
     ponto.Z:=ponto.Z+(3*t2 - 3*t3)*getPonto(3).Z;
     ponto.Z:=ponto.Z+(t3)*getPonto(4).Z;

     result := ponto;

end;

procedure TBezier.escalona(origem: boolean; sx, sy, sz: real);
var
   temp     : TPonto;
   i        : integer;
   dx,dy,dz : real;
begin
   if not origem then begin
      dx := 0;
      dy := 0;
      dz := 0;
      for i := 1 to npontos do begin
         temp := getPonto(i);
         dx := dx + temp.x;
         dy := dy + temp.y;
         dz := dz + temp.z;
      end;
      dx := dx/npontos;
      dy := dy/npontos;
      dz := dz/npontos;
      translada(-dx,-dy,-dz);
   end;
   for i:=1 to npontos do begin
      temp := getPonto(i);
      temp.escalona(true,sx,sy,sz);
   end;
   if not origem then
      translada(dx,dy,dz);
end;

procedure TBezier.rotacao(tipoRotacao, eixoRotacao, angulo: integer);
var
  temp     : TPonto;
  i        : integer;
  dx,dy,dz : real;
begin
   if tipoRotacao=2 then begin
      dx := 0;
      dy := 0;
      dz := 0;
      for i := 1 to npontos do begin
         temp := getPonto(i);
         dx := dx + temp.x;
         dy := dy + temp.y;
         dz := dz + temp.z;
      end;
      dx := dx/npontos;
      dy := dy/npontos;
      dz := dz/npontos;
      translada(-dx,-dy,-dz);
   end;
   for i := 1 to npontos do begin
      temp := getPonto(i);
      temp.rotacao(0,eixoRotacao,angulo);
   end;
   if tipoRotacao = 2 then translada(dx,dy,dz);
end;

procedure TBezier.translada(dx, dy, dz: real);
var
   temp : TPonto;
   i : integer;
begin
   for i:=1 to npontos do begin
      temp := getPonto(i);
      temp.translada(dx,dy,dz);
   end;
end;

end.
