unit uObjeto3d;

interface

uses uMatriz, Math, UObjeto,uTipo, uPonto;

type
  TObjeto3d = class(TObjeto)
  private
    procedure escalona3d(sx, sy, sz: real);
    procedure rotacaoX(angulo: integer);
    procedure rotacaoY(angulo: integer);
    procedure rotacaoZ(angulo: integer);
  protected
     lig : TMatriz;
     pontos : TMatriz;
  public
    procedure setaPontos(v: wf);
    procedure translada(dx,dy,dz: real); override;
    procedure rotacao(tipoRotacao,eixoRotacao,angulo : integer); override;
    procedure escalona(origem: boolean; sx,sy,sz: real); override;
    function getPonto(i : integer) : TPonto;
    function getNPontos() : integer;
    function getArestas(i : integer) : TMatriz;
    destructor destrua_se;
    constructor crie_se(_nome: String; npontos: integer);
  end;

implementation

constructor TObjeto3d.crie_se(_nome:String; npontos: integer);
begin
  nome:=_nome;
  pontos:=TMatriz.crie_se(npontos,3);
  lig:=TMatriz.crie_se(npontos,7);
end;

destructor TObjeto3d.destrua_se;
begin
end;

procedure TObjeto3d.translada(dx, dy, dz: real);
var
 i: integer;
begin
 for i:=1 to pontos.nlinhas do begin
   pontos.setaValor(i,1,pontos.retornaValor(i,1)+dx);
   pontos.setaValor(i,2,pontos.retornaValor(i,2)+dy);
   pontos.setaValor(i,3,pontos.retornaValor(i,3)+dz);
 end;
end;

procedure TObjeto3d.escalona3d(sx, sy, sz: real);
var
 i: integer;
begin

 for i:=1 to pontos.nlinhas do begin
   pontos.setaValor(i,1,pontos.retornaValor(i,1)*sx);
   pontos.setaValor(i,2,pontos.retornaValor(i,2)*sy);
   pontos.setaValor(i,3,pontos.retornaValor(i,3)*sz);
 end;

end;

procedure TObjeto3d.rotacaoX(angulo: integer);
var
 y,z : real;
 i : integer;
begin

 for i:=1 to pontos.nlinhas do begin
   y:=pontos.retornaValor(i,2);
   z:=pontos.retornaValor(i,3);
   pontos.setaValor(i,2,  y*cos(DegToRad(angulo)) - z*sin(DegToRad(angulo)) );
   pontos.setaValor(i,3, y*sin(DegToRad(angulo))+ z*cos(DegToRad(angulo)) );
 end;

end;

procedure TObjeto3d.rotacaoY(angulo: integer);
var
 x,z : real;
 i : integer;
begin

 for i:=1 to pontos.nlinhas do begin
   x:=pontos.retornaValor(i,1);
   z:=pontos.retornaValor(i,3);
   pontos.setaValor(i,1, x*cos(DegToRad(angulo)) + z*sin(DegToRad(angulo)) );
   pontos.setaValor(i,3, -x*sin(DegToRad(angulo)) + z*cos(DegToRad(angulo)) );
 end;

end;

procedure TObjeto3d.rotacaoZ(angulo: integer);
var
 x,y : real;
 i : integer;
begin

 for i:=1 to pontos.nlinhas do begin
   x:=pontos.retornaValor(i,1);
   y:=pontos.retornaValor(i,2);
   pontos.setaValor(i,1, x*cos(DegToRad(angulo)) - y*sin(DegToRad(angulo)) );
   pontos.setaValor(i,2, x*sin(DegToRad(angulo)) + y*cos(DegToRad(angulo)) );
 end;

end;

procedure TObjeto3d.setaPontos(v : wf);
var
 i,j: integer;
begin
  for i:=1 to pontos.nlinhas do begin
    for j:=1 to 3 do pontos.setaValor(i,j,v[i,j]);
    for j:=4 to 9 do lig.setaValor(i,j-3,v[i,j]);
  end;
end;

function TObjeto3d.getArestas(i: integer): TMatriz;
var
   n,j : integer;
   nMatriz : TMatriz;
begin
   n:=0;
   for j:= 1 to 6 do begin
    if lig.retornaValor(i,j) = 0 then break
    else inc(n);
   end;
   nMatriz := TMatriz.crie_se(1,n);
   for j:=1 to n do
      nMatriz.setaValor(1,j,lig.retornaValor(i,j));
   result := nMatriz;
end;

function TObjeto3d.getNPontos: integer;
begin
   result := pontos.nlinhas;
end;

function TObjeto3d.getPonto(i: integer): TPonto;
var
   pontoRet : TPonto;
begin
   pontoRet := TPonto.crie_se('',pontos.retornaValor(i,1),pontos.retornaValor(i,2),pontos.retornaValor(i,3));
   result := pontoRet;
end;

procedure TObjeto3d.escalona(origem: boolean; sx,sy,sz: real);
var
   dx,dy,dz : real;
   n,i : integer;
begin
   if not origem then begin
     n := pontos.ncolunas;
     dx := 0;
     dy := 0;
     dz := 0;
     for i := 1 to n do begin
        dx := dx + pontos.retornaValor(i,1);
        dy := dy + pontos.retornaValor(i,2);
        dz := dz + pontos.retornaValor(i,3);
     end;
     dx := dx/n;
     dy := dy/n;
     dz := dz/n;
     translada(-dx,-dy,-dz);
   end;
   escalona3d(sx,sy,sz);
   if not origem then translada(dx,dy,dz);
end;

procedure TObjeto3d.rotacao(tipoRotacao,eixoRotacao,angulo: integer);
var
   n,i : integer;
   dx,dy,dz : real;
begin
  if tipoRotacao = 2 then begin //sobre o centro do objeto
     n := pontos.ncolunas;
     dx := 0;
     dy := 0;
     dz := 0;
     for i := 1 to n do begin
        dx := dx + pontos.retornaValor(i,1);
        dy := dy + pontos.retornaValor(i,2);
        dz := dz + pontos.retornaValor(i,3);
     end;
     dx := dx/n;
     dy := dy/n;
     dz := dz/n;
     translada(-dx,-dy,-dz);
  end;
  if(eixoRotacao = 0) then
     rotacaoX(angulo)
  else
     if(eixoRotacao = 1)then
        rotacaoY(angulo)
     else
        rotacaoZ(angulo);
  if tipoRotacao = 2 then translada(dx,dy,dz);
end;

end.
