unit uSpline;

interface

   uses UObjeto, UPonto, Dialogs, Classes,uMatriz;


   Type
     Pponto = ^aponto;
       aponto = record
         ponto : TPonto;
   end;

   type TSpline=class(TObjeto)
  private
     matrizB : TMatriz;
     vnpontos : integer;
     procedure setPonto(i: integer; x, y, z: real);
    procedure calculaDiferencas(n: integer; a, b, c, d: real; var f, df,
      d2f, d3f: real);
    procedure CalculaCoeficientes(var matrizCX, matrizCY, matrizCZ: Tmatriz;
      n: integer);
    function retornaMatrizB: TMatriz;
   protected
     Spline : Tlist;
   public
      function MatrizPontosSpline(segCurva, n: integer): TMatriz;
      function getPonto(i: integer): TPonto;
      procedure adicionaPonto(_x, _y, _z: integer);
      procedure translada(dx,dy,dz:real); override;
      procedure escalona(origem: boolean; sx, sy, sz: real);override;
      procedure rotacao(tipoRotacao, eixoRotacao, angulo: integer); override;
      constructor crie_se(_nome:String);
      destructor destrua_se;

   published
    property npontos: integer read vnpontos write vnpontos;

  end;

implementation

procedure TSpline.adicionaPonto(_x, _y, _z: integer);
var
 ponto: Pponto;
begin
   New(ponto);
   ponto^.ponto:=TPonto.crie_se(nome,_x,_y,_z);
   Spline.Add(ponto);
   npontos:=npontos+1;
end;

constructor TSpline.crie_se(_nome:String);
begin
   tipo:='Spline';
   nome:=_nome;
   Spline:=TList.Create;
   npontos:=0;
   matrizB:=retornaMatrizB;
end;

destructor TSpline.destrua_se;
begin
end;


function TSpline.getPonto(i: integer): TPonto;
var
 ponto : Pponto;
begin
    New(ponto);
    ponto:= Spline.items[i-1];
    result:=ponto^.ponto;
end;

procedure TSpline.setPonto(i:integer;x,y,z : real);
var
 ponto : Pponto;
begin
   New(ponto);
   ponto^.ponto:=TPonto.crie_se(nome,x,y,z);
   Spline.items[i-1]:=ponto;
end;

procedure TSpline.calculaDiferencas(n: integer; a,b,c,d: real; var f, df, d2f, d3f: real);
var
  delta, delta2, delta3 : real;
begin
  delta := 1/n;
  delta2 := delta * delta;
  delta3 := delta2 * delta;

  f := d;
  df := a*delta3 + b*delta2 + c*delta;
  d2f := 6*a*delta3 + 2*b*delta2;
  d3f := 6*a*delta3;

end;

procedure TSpline.CalculaCoeficientes(var matrizCX, matrizCY, matrizCZ: TMatriz; n: integer);
var
   matrizX, matrizY, matrizZ : TMatriz;
begin

    matrizX:=TMatriz.crie_se(4,1);
    matrizX.setaValor(1,1,getPonto(n).x);
    matrizX.setaValor(2,1,getPonto(n + 1).x);
    matrizX.setaValor(3,1,getPonto(n + 2).x);
    matrizX.setaValor(4,1,getPonto(n + 3).x);

    matrizY:=TMatriz.crie_se(4,1);
    matrizY.setaValor(1,1,getPonto(n).y);
    matrizY.setaValor(2,1,getPonto(n + 1).y);
    matrizY.setaValor(3,1,getPonto(n + 2).y);
    matrizY.setaValor(4,1,getPonto(n + 3).y);

    matrizZ:=TMatriz.crie_se(4,1);
    matrizZ.setaValor(1,1,getPonto(n).z);
    matrizZ.setaValor(2,1,getPonto(n + 1).z);
    matrizZ.setaValor(3,1,getPonto(n + 2).z);
    matrizZ.setaValor(4,1,getPonto(n + 3).z);

    matrizCX:=TMatriz.crie_se(1,4);
    matrizCx:=TMatriz.multiplicaMatriz(matrizB, matrizX);

    matrizCY:=TMatriz.crie_se(1,4);
    matrizCY:=TMatriz.multiplicaMatriz(matrizB, matrizY);

    matrizCZ := TMatriz.crie_se(1,4);
    matrizCZ := TMatriz.multiplicaMatriz(matrizB, matrizZ);
end;

function TSpline.MatrizPontosSpline(segCurva, n: integer): TMatriz;
var
  i: integer;
  matrizCX, matrizCY, matrizCZ, matrizResult: TMatriz;
  xi, Dx, D2x, D3x: real;
  yi, Dy, D2y, D3y: real;
  zi, Dz, D2z, D3z: real;
begin
   CalculaCoeficientes(matrizCX, matrizCY, matrizCZ, segCurva);
   CalculaDiferencas(n,matrizCX.retornaValor(1,1), matrizCX.retornaValor(1,2), matrizCX.retornaValor(1,3), matrizCX.retornaValor(1,4), xi,Dx,D2x,D3x);
   CalculaDiferencas(n,matrizCY.retornaValor(1,1), matrizCY.retornaValor(1,2), matrizCY.retornaValor(1,3), matrizCY.retornaValor(1,4), yi,Dy,D2y,D3y);
   CalculaDiferencas(n,matrizCZ.retornaValor(1,1), matrizCZ.retornaValor(1,2), matrizCZ.retornaValor(1,3), matrizCZ.retornaValor(1,4), zi,Dz,D2z,D3z);

   matrizResult:=TMatriz.crie_se(3,n+1);
   matrizResult.setaValor(1,1,xi);
   matrizResult.setaValor(2,1,yi);
   matrizResult.setaValor(3,1,zi);
   for i:= 2 to n + 1 do
   begin
     xi:= xi + Dx;
     Dx:= Dx + D2x;
     D2x:= D2x + D3x;

     yi := yi + Dy;
     Dy := Dy + D2y;
     D2y:= D2y + D3y;

     zi  := zi  + Dz;
     Dz  := Dz  + D2z;
     D2z := D2z + D3z;

     matrizResult.setaValor(1,i,xi);
     matrizResult.setaValor(2,i,yi);
     matrizResult.setaValor(3,i,zi);
   end;

   result:=matrizResult;
end;

function TSpline.retornaMatrizB: TMatriz;
var
 matrizB: TMatriz;
begin
    //Matriz B-Spline
    matrizB:=TMatriz.crie_se(4,4);

    matrizB.setaValor(1,1,-1/6);
    matrizB.setaValor(1,2,3/6);
    matrizB.setaValor(1,3,-3/6);
    matrizB.setaValor(1,4,1/6);

    matrizB.setaValor(2,1,3/6);
    matrizB.setaValor(2,2,-6/6);
    matrizB.setaValor(2,3,3/6);
    matrizB.setaValor(2,4,0);

    matrizB.setaValor(3,1,-3/6);
    matrizB.setaValor(3,2,0);
    matrizB.setaValor(3,3,3/6);
    matrizB.setaValor(3,4,0);

    matrizB.setaValor(4,1,1/6);
    matrizB.setaValor(4,2,4/6);
    matrizB.setaValor(4,3,1/6);
    matrizB.setaValor(4,4,0);
    result:=matrizB;

end;

procedure TSpline.escalona(origem: boolean; sx, sy, sz: real);
var
   temp     : TPonto;
   i        : integer;
   dx,dy,dz : real;
begin
   if not origem then begin
      dx := 0;
      dy := 0;
      dz := 0;
      for i := 1 to npontos do begin
         temp := getPonto(i);
         dx := dx + temp.x;
         dy := dy + temp.y;
         dz := dz + temp.z;
      end;
      dx := dx/npontos;
      dy := dy/npontos;
      dz := dz/npontos;
      translada(-dx,-dy,-dz);
   end;
   for i:=1 to npontos do begin
      temp := getPonto(i);
      temp.escalona(true,sx,sy,sz);
   end;
   if not origem then
      translada(dx,dy,dz);
end;

procedure TSpline.rotacao(tipoRotacao, eixoRotacao, angulo: integer);
var
  temp     : TPonto;
  i        : integer;
  dx,dy,dz : real;
begin
   if tipoRotacao=2 then begin
      dx := 0;
      dy := 0;
      dz := 0;
      for i := 1 to npontos do begin
         temp := getPonto(i);
         dx := dx + temp.x;
         dy := dy + temp.y;
         dz := dz + temp.z;
      end;
      dx := dx/npontos;
      dy := dy/npontos;
      dz := dz/npontos;
      translada(-dx,-dy,-dz);
   end;
   for i := 1 to npontos do begin
      temp := getPonto(i);
      temp.rotacao(0,eixoRotacao,angulo);
   end;
   if tipoRotacao = 2 then translada(dx,dy,dz);
end;

procedure TSpline.translada(dx, dy, dz: real);
var
   temp : TPonto;
   i : integer;
begin
   for i:=1 to npontos do begin
      temp := getPonto(i);
      temp.translada(dx,dy,dz);
   end;
end;

end.
