unit uSuperficie;

interface

uses uObjeto,uPonto, uMatriz, Dialogs, SysUtils, Classes;

type
  TSuperficie = class(TObjeto)
  private
    function retornaMatriz(aresta: integer): TMatriz;
    function calculeProxPonto(s, t: real): TPonto;
  protected
     pontos : TMatriz;
  public
    tipo : String;
    function retornaPonto(pos: integer) : TPOnto;
    procedure translada(dx,dy,dz:real); override;
    procedure escalona(origem: boolean; sx, sy, sz: real);override;
    procedure rotacao(tipoRotacao, eixoRotacao, angulo: integer); override;
    procedure setaPonto(n : integer; x, y, z: real);
    constructor crie_se(_nome:String);
    destructor destrua_se;
    function matrizRetas(n,m,k : integer):TList;
  protected
    function retornaMatrizB : TMatriz;


  published
    property nome: string read vnome write vnome;
  end;

implementation

constructor TSuperficie.crie_se(_nome:String);
begin
  nome:=_nome;
  pontos:=TMatriz.crie_se(16,3);
end;

destructor TSuperficie.destrua_se;
begin
end;

function TSuperficie.matrizRetas(n, m, k: integer): TList;
var
    resp, linha : TList;
    s,t,passo, passoS, passoT : real;
    p1,p2 : TPonto;
begin

   resp := TList.Create;

        passo  := 1.0 / k*10;
        passoS := 1.0 / n;
        passoT := 1.0 / m;

        s := 0;
        while s<=1 do begin
            p1 := calculeProxPonto(s, 0);
            t := passo;
            linha := TList.create();
            linha.add(p1);
            while t<=1 do begin
               p2 := calculeProxPonto(s, t);
               p1 := p2;
               linha.add(p1);
               t := t + passo;
            end;
            resp.Add(linha);
            s := s + passoS;
        end;

        t := 0;
        while t<=1 do begin
            p1 := calculeProxPonto(0, t);
            s:=passo;
            linha := TList.create();
            linha.add(p1);
            while s<=1 do begin
               p2 := calculeProxPonto(s, t);
               p1 := p2;
               linha.add(p1);
               s := s + passo;
            end;
            resp.Add(linha);
            t := t + passoT;
        end;

        Result := resp;
end;

procedure TSuperficie.setaPonto(n: integer; x, y, z: real);
begin
   pontos.setaValor(n,1,x);
   pontos.setaValor(n,2,y);
   pontos.setaValor(n,3,z);
end;

procedure TSuperficie.rotacao(tipoRotacao, eixoRotacao, angulo: integer);
var
  temp     : TPonto;
  i, npontos        : integer;
  dx,dy,dz : real;
begin
   npontos := pontos.nlinhas;
   if tipoRotacao=2 then begin
      dx := 0;
      dy := 0;
      dz := 0;
      for i := 1 to npontos do begin
         dx := dx + pontos.retornaValor(i,1);
         dy := dy + pontos.retornaValor(i,2);
         dz := dz + pontos.retornaValor(i,3);
      end;
      dx := dx/npontos;
      dy := dy/npontos;
      dz := dz/npontos;
      translada(-dx,-dy,-dz);
   end;
   temp := TPonto.crie_se('',0,0,0);
   for i := 1 to npontos do begin
      temp.x := pontos.retornaValor(i,1);
      temp.y := pontos.retornaValor(i,2);
      temp.z := pontos.retornaValor(i,3);
      temp.rotacao(0,eixoRotacao,angulo);
      self.setaPonto(i,temp.x,temp.y,temp.z);
   end;
   if tipoRotacao = 2 then translada(dx,dy,dz);
end;

procedure TSuperficie.translada(dx, dy, dz: real);
var i : integer;
    nx, ny, nz : real;
begin
   for i:=1 to pontos.nlinhas do begin
      nx := pontos.retornaValor(i,1) + dx;
      ny := pontos.retornaValor(i,2) + dy;
      nz := pontos.retornaValor(i,3) + dz;
      self.setaPonto(i,nx,ny,nz);
   end;
end;

procedure TSuperficie.escalona(origem: boolean; sx, sy, sz: real);
var
   i,nPontos        : integer;
   dx,dy,dz,nx,ny,nz : real;
begin
   npontos := self.pontos.nlinhas;
   if not origem then begin
      dx := 0;
      dy := 0;
      dz := 0;
      for i := 1 to npontos do begin
         dx := dx + pontos.retornaValor(i,1);
         dy := dy + pontos.retornaValor(i,2);
         dz := dz + pontos.retornaValor(i,3);
      end;
      dx := dx/npontos;
      dy := dy/npontos;
      dz := dz/npontos;
      translada(-dx,-dy,-dz);
   end;
   for i:=1 to npontos do begin
      nx := pontos.retornaValor(i,1) * sx;
      ny := pontos.retornaValor(i,2) * sy;
      nz := pontos.retornaValor(i,3) * sz;
      self.setaPonto(i,nx,ny,nz);
   end;
   if not origem then
      translada(dx,dy,dz);
end;


function TSuperficie.retornaMatrizB: TMatriz;
var
 matrizB: TMatriz;
begin

    //Matriz B-Spline
    matrizB:=TMatriz.crie_se(4,4);

    matrizB.setaValor(1,1,-1/6);
    matrizB.setaValor(1,2,3/6);
    matrizB.setaValor(1,3,-3/6);
    matrizB.setaValor(1,4,1/6);

    matrizB.setaValor(2,1,3/6);
    matrizB.setaValor(2,2,-6/6);
    matrizB.setaValor(2,3,3/6);
    matrizB.setaValor(2,4,0);

    matrizB.setaValor(3,1,-3/6);
    matrizB.setaValor(3,2,0);
    matrizB.setaValor(3,3,3/6);
    matrizB.setaValor(3,4,0);

    matrizB.setaValor(4,1,1/6);
    matrizB.setaValor(4,2,4/6);
    matrizB.setaValor(4,3,1/6);
    matrizB.setaValor(4,4,0);
    result := matrizB;

end;

function TSuperficie.retornaMatriz(aresta: integer): TMatriz;
var resp : TMatriz;
    i,j : integer;
begin
   resp := TMatriz.crie_se(4,4);
   for i := 1 to 4 do
      for j := 3 downto 0 do
         resp.setaValor(i,j+1,self.pontos.retornaValor(i*4-j,aresta));
   Result := resp;
end;

function TSuperficie.retornaPonto(pos : integer) : TPonto;
var
   resp : TPOnto;
begin
   resp := TPonto.crie_se('Nome',0,0,0);
   resp.x := pontos.retornaValor(pos+1,1);
   resp.y := pontos.retornaValor(pos+1,2);
   resp.z := pontos.retornaValor(pos+1,3);
   Result := resp;
end;

function TSuperficie.calculeProxPonto(s , t: real) : TPonto;
var
   s2, s3, t2,t3 : real;
   d6,d7,d8,d9,d10,d11,d12,d13 : real;
   nPonto : TPonto;
begin
        nPonto := TPonto.crie_se('Nome',0,0,0);
        s2 := s * s;
        s3 := s2 * s;
        t2 := t * t;
        t3 := t2 * t;

        d6 := (((-s3 + 3 * s2) - 3 * s) + 1.0) / 6;
        d7 := ((3 * s3 - 6 * s2) + 4) / 6;
        d8 := (-3 * s3 + 3 * s2 + 3 * s + 1.0) / 6;
        d9 := s3 / 6;
        d10 := (((-t3 + 3 * t2) - 3 * t) + 1.0) / 6;
        d11 := ((3 * t3 - 6 * t2) + 4) / 6;
        d12 := (-3 * t3 + 3 * t2 + 3 * t + 1.0) / 6;
        d13 := t3 / 6;

        nPonto.x := (((d6 * retornaPonto(0).x + d7 * retornaPonto(4).x + d8 * retornaPonto(8).x +
        	d9 * retornaPonto(12).x) * d10 + (d6 * retornaPonto(1).x +
        	d7 * retornaPonto(5).x + d8 * retornaPonto(9).x +
        	d9 * retornaPonto(13).x) * d11 +
        	(d6 * retornaPonto(2).x + d7 * retornaPonto(6).x +
        	d8 * retornaPonto(10).x +
        	d9 * retornaPonto(14).x) * d12 +
        	(d6 * retornaPonto(3).x + d7 * retornaPonto(7).x +
        	d8 * retornaPonto(11).x +
        	d9 * retornaPonto(15).x) * d13));

        nPonto.y := (((d6 * retornaPonto(0).y + d7 * retornaPonto(4).y + d8 * retornaPonto(8).y +
        	d9 * retornaPonto(12).y) * d10 + (d6 * retornaPonto(1).y +
        	d7 * retornaPonto(5).y + d8 * retornaPonto(9).y +
        	d9 * retornaPonto(13).y) * d11 +
        	(d6 * retornaPonto(2).y + d7 * retornaPonto(6).y +
        	d8 * retornaPonto(10).y +
        	d9 * retornaPonto(14).y) * d12 +
        	(d6 * retornaPonto(3).y + d7 * retornaPonto(7).y +
        	d8 * retornaPonto(11).y +
        	d9 * retornaPonto(15).y) * d13));

        nPonto.z := (((d6 * retornaPonto(0).z + d7 * retornaPonto(4).z + d8 * retornaPonto(8).z +
        	d9 * retornaPonto(12).z) * d10 + (d6 * retornaPonto(1).z +
        	d7 * retornaPonto(5).z + d8 * retornaPonto(9).z +
        	d9 * retornaPonto(13).z) * d11 +
        	(d6 * retornaPonto(2).z + d7 * retornaPonto(6).z +
        	d8 * retornaPonto(10).z +
        	d9 * retornaPonto(14).z) * d12 +
        	(d6 * retornaPonto(3).z + d7 * retornaPonto(7).z +
        	d8 * retornaPonto(11).z +
        	d9 * retornaPonto(15).z) * d13));

        Result := nPonto;

end;


end.
